#include "stdafx.h"
#include "d3dclass.h"
#include "cameraclass.h"
#include "modelclass.h"
#include "textureshaderclass.h"
#include "graphicsclass.h"


GraphicsClass::GraphicsClass()
{
}


GraphicsClass::GraphicsClass(const GraphicsClass& other)
{
}


GraphicsClass::~GraphicsClass()
{
}


bool GraphicsClass::Initialize(int screenWidth, int screenHeight, HWND hwnd)
{
	// Direct3D ü 
	m_Direct3D = new D3DClass;
	if(!m_Direct3D)
	{
		return false;
	}

	// Direct3D ü ʱȭ
	if(!m_Direct3D->Initialize(screenWidth, screenHeight, VSYNC_ENABLED, hwnd, FULL_SCREEN, SCREEN_DEPTH, SCREEN_NEAR))
	{
		MessageBox(hwnd, L"Could not initialize Direct3D.", L"Error", MB_OK);
		return false;
	}

	// m_Camera ü 
	m_Camera = new CameraClass;
	if (!m_Camera)
	{
		return false;
	}

	// ī޶  
	m_Camera->SetPosition(0.0f, 0.0f, -3.0f);

	// m_Model ü 
	m_Model = new ModelClass;
	if (!m_Model)
	{
		return false;
	}

	// m_Model ü ʱȭ
	if (!m_Model->Initialize(m_Direct3D->GetDevice(), L"../Dx11Demo_37/data/seafloor.dds"))
	{
		MessageBox(hwnd, L"Could not initialize the model object.", L"Error", MB_OK);
		return false;
	}

	// m_TextureShader ü 
	m_TextureShader = new TextureShaderClass;
	if (!m_TextureShader)
	{
		return false;
	}

	// m_TextureShader ü ʱȭ
	if (!m_TextureShader->Initialize(m_Direct3D->GetDevice(), hwnd))
	{
		MessageBox(hwnd, L"Could not initialize the texture shader object.", L"Error", MB_OK);
		return false;
	}

	return true;
}


void GraphicsClass::Shutdown()
{
	// m_TextureShader ü ȯ
	if (m_TextureShader)
	{
		m_TextureShader->Shutdown();
		delete m_TextureShader;
		m_TextureShader = 0;
	}

	// m_Model ü ȯ
	if (m_Model)
	{
		m_Model->Shutdown();
		delete m_Model;
		m_Model = 0;
	}

	// m_Camera ü ȯ
	if (m_Camera)
	{
		delete m_Camera;
		m_Camera = 0;
	}

	// Direct3D ü ȯ
	if (m_Direct3D)
	{
		m_Direct3D->Shutdown();
		delete m_Direct3D;
		m_Direct3D = 0;
	}
}


bool GraphicsClass::Frame()
{
	// ׷  ó
	return Render();
}


bool GraphicsClass::Render()
{
	//  ׸  ۸ ϴ
	m_Direct3D->BeginScene(0.0f, 0.0f, 0.0f, 1.0f);

	// ī޶ ġ    մϴ
	m_Camera->Render();

	// ī޶  d3d ü ,     ɴϴ
	XMMATRIX worldMatrix, viewMatrix, projectionMatrix;
	m_Direct3D->GetWorldMatrix(worldMatrix);
	m_Camera->GetViewMatrix(viewMatrix);
	m_Direct3D->GetProjectionMatrix(projectionMatrix);

	//  ؽ ε ۸ ׷  ο ġϿ  غմϴ.
	m_Model->Render(m_Direct3D->GetDeviceContext());

	// ؽ ̴ Ͽ  մϴ.
	if (!m_TextureShader->Render(m_Direct3D->GetDeviceContext(), m_Model->GetVertexCount(), m_Model->GetInstanceCount(), worldMatrix, viewMatrix, 
									 projectionMatrix, m_Model->GetTexture()))
	{
		return false;
	}

	//   ȭ鿡 մϴ
	m_Direct3D->EndScene();

	return true;
}