#include "stdafx.h"
#include "TextureClass.h"
#include "modelclass.h"


ModelClass::ModelClass()
{
}


ModelClass::ModelClass(const ModelClass& other)
{
}


ModelClass::~ModelClass()
{
}


bool ModelClass::Initialize(ID3D11Device* device, const WCHAR* textureFilename)
{
	//   ε ۸ ʱȭմϴ.
	if (!InitializeBuffers(device))
	{
		return false;
	}

	//   ؽó εմϴ.
	return LoadTexture(device, textureFilename);
}


void ModelClass::Shutdown()
{
	//  ؽĸ ȯմϴ.
	ReleaseTexture();

	// ؽ  ε ۸ մϴ.
	ShutdownBuffers();
}


void ModelClass::Render(ID3D11DeviceContext* deviceContext)
{
	// ׸⸦ غϱ  ׷  ο  ε ۸ ϴ.
	RenderBuffers(deviceContext);
}


int ModelClass::GetVertexCount()
{
	return m_vertexCount;
}


int ModelClass::GetInstanceCount()
{
	return m_instanceCount;
}


ID3D11ShaderResourceView* ModelClass::GetTexture()
{
	return m_Texture->GetTexture();
}


bool ModelClass::InitializeBuffers(ID3D11Device* device)
{
	//  迭   մϴ.
	m_vertexCount = 3;

	//  迭 ϴ.
	VertexType* vertices = new VertexType[m_vertexCount];
	if (!vertices)
	{
		return false;
	}

	//  迭  մϴ.
	vertices[0].position = XMFLOAT3(-1.0f, -1.0f, 0.0f);  // Bottom left.
	vertices[0].texture = XMFLOAT2(0.0f, 1.0f);

	vertices[1].position = XMFLOAT3(0.0f, 1.0f, 0.0f);  // Top middle.
	vertices[1].texture = XMFLOAT2(0.5f, 0.0f);

	vertices[2].position = XMFLOAT3(1.0f, -1.0f, 0.0f);  // Bottom right.
	vertices[2].texture = XMFLOAT2(1.0f, 1.0f);

	//    ü մϴ.
	D3D11_BUFFER_DESC vertexBufferDesc;
	vertexBufferDesc.Usage = D3D11_USAGE_DEFAULT;
	vertexBufferDesc.ByteWidth = sizeof(VertexType) * m_vertexCount;
	vertexBufferDesc.BindFlags = D3D11_BIND_VERTEX_BUFFER;
	vertexBufferDesc.CPUAccessFlags = 0;
	vertexBufferDesc.MiscFlags = 0;
	vertexBufferDesc.StructureByteStride = 0;

	// subresource   Ϳ  ͸ մϴ.
	D3D11_SUBRESOURCE_DATA vertexData;
	vertexData.pSysMem = vertices;
	vertexData.SysMemPitch = 0;
	vertexData.SysMemSlicePitch = 0;

	//   ۸ ϴ.
	if (FAILED(device->CreateBuffer(&vertexBufferDesc, &vertexData, &m_vertexBuffer)))
	{
		return false;
	}

	//  ؽ ۰ ǰε  ؽ 迭 Ѵ.
	delete [] vertices;
	vertices = 0;

	// 迭 νϽ  մϴ.
	m_instanceCount = 4;

	// νϽ 迭 ϴ.
	InstanceType* instances = new InstanceType[m_instanceCount];
	if(!instances)
	{
		return false;
	}

	// ͷ νϽ 迭 εմϴ.
	instances[0].position = XMFLOAT3(-1.5f, -1.5f, 5.0f);
	instances[1].position = XMFLOAT3(-1.5f,  1.5f, 5.0f);
	instances[2].position = XMFLOAT3( 1.5f, -1.5f, 5.0f);
	instances[3].position = XMFLOAT3( 1.5f,  1.5f, 5.0f);

	// νϽ   մϴ.
	D3D11_BUFFER_DESC instanceBufferDesc;
    instanceBufferDesc.Usage = D3D11_USAGE_DEFAULT;
    instanceBufferDesc.ByteWidth = sizeof(InstanceType) * m_instanceCount;
    instanceBufferDesc.BindFlags = D3D11_BIND_VERTEX_BUFFER;
    instanceBufferDesc.CPUAccessFlags = 0;
    instanceBufferDesc.MiscFlags = 0;
	instanceBufferDesc.StructureByteStride = 0;

	//  ҽ  νϽ Ϳ  ͸ մϴ.
	D3D11_SUBRESOURCE_DATA instanceData;
    instanceData.pSysMem = instances;
	instanceData.SysMemPitch = 0;
	instanceData.SysMemSlicePitch = 0;

	// νϽ ۸ ϴ.
	if(FAILED(device->CreateBuffer(&instanceBufferDesc, &instanceData, &m_instanceBuffer)))
	{
		return false;
	}

	// νϽ ۰ ǰεǾǷ νϽ 迭 մϴ.
	delete [] instances;
	instances = 0;

	return true;
}


void ModelClass::ShutdownBuffers()
{
	// νϽ ۸ մϴ.
	if(m_instanceBuffer)
	{
		m_instanceBuffer->Release();
		m_instanceBuffer = 0;
	}

	//  ۸ մϴ.
	if (m_vertexBuffer)
	{
		m_vertexBuffer->Release();
		m_vertexBuffer = 0;
	}
}


void ModelClass::RenderBuffers(ID3D11DeviceContext* deviceContext)
{
	//  ° Ʈ̵带 մϴ.
	unsigned int strides[2] = { sizeof(VertexType), sizeof(InstanceType) };
	unsigned int offsets[2] = { 0, 0 };

	//  迭  ۿ νϽ ۷ մϴ.
	ID3D11Buffer* bufferPointers[2] = { m_vertexBuffer, m_instanceBuffer };

	//    ֵ Է   ۸ Ȱ մϴ.
	deviceContext->IASetVertexBuffers(0, 2, bufferPointers, strides, offsets);

    //   ۿ Ǿϴ Ƽ  մϴ. 쿡 ﰢԴϴ.
	deviceContext->IASetPrimitiveTopology(D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST);
}


bool ModelClass::LoadTexture(ID3D11Device* device, const WCHAR* filename)
{
	// ؽó Ʈ Ѵ.
	m_Texture = new TextureClass;
	if (!m_Texture)
	{
		return false;
	}

	// ؽó Ʈ ʱȭѴ.
	return m_Texture->Initialize(device, filename);
}


void ModelClass::ReleaseTexture()
{
	// ؽó Ʈ Ѵ.
	if (m_Texture)
	{
		m_Texture->Shutdown();
		delete m_Texture;
		m_Texture = 0;
	}
}